/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* gameplay.h */

#ifndef _GAMEPLAY_H
#define _GAMEPLAY_H

#include "data.h"

/* Main game-playing routine */
void PlayGame (GAME *game, PLAYER *player);

/* Save the state of the current game
   OUTPUT: Returns FALSE on failure */
unsigned char SaveGame(GAME *game, PLAYER *player);

/* Restores the state of a game
   OUTPUT: Returns FALSE on failure */
unsigned char LoadGame(GAME *game, PLAYER *player);

/* Get user keypress and act on it
   OUTPUT: Returns FALSE if key pressed indicates the user wants to quit */
unsigned char ProcessInGameKey (unsigned char key, GAME *game, PLAYER *player);

/* User interface for setting the game options before a game.  Sets the game
   options in the game struct.  Returns TRUE if the game is to begin, or
   FALSE if the player wants to exit the program. */
unsigned char GetGameOptions (GAME *game);

/* Check is proposed move is legal
   Checks whether the user's proposed card movement follows the rules of the
   game.  This function is mainly used for the UI to prevent the user from
   making an illegal move.
   oldPos: The existing position of the card
   newPos: The proposed location to move the card
   playerNo: The player who is making the turn
   game, player: Game and player structures
   OUTPUT: Returns TRUE if it's legal, FALSE if it's illegal */
unsigned char IsLegalMove (unsigned char oldPos, unsigned char newPos,
unsigned char playerNo, GAME *game, PLAYER *player);

#endif
