/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* graphics.c
   Functions dealing with game display */

#include <compat.h>
#include <graph.h>
#include <stdio.h>
#include <sprites.h>
#include <system.h>

#include "skipbo.h"
#include "graphics.h"

///// Text string constants ////////////////////////////////////////////////////

/* Basic info */
static const char *str_author = COMMENT_PROGRAM_NAME " " COMMENT_VERSION_STRING
	" by Travis Evans";
static const char *str_gametype[] = {"STANDARD", "QUICK"};
static const char *str_gameDifficulty[] = {"EASY", "NORMAL", "HARD"};
static const char *str_player = "PLYR";
static const char *str_playerWins = "PLAYER %c WINS!";
static const char *str_noDrawCards = "No cards left for draw pile!";
static const char *str_gameOver = "GAME OVER";
static const char *str_playerChar = "ABCD";
static const char *str_pile = "Pile";
static const char *str_pileNames[] = {"Building","Stock","Discard","Hand"};
static const char *str_shuffling = "SHUFFLING";

/* Key help */
static const char *str_keyHelp1 =
	"\x15\x16\x17\x18: Move \xB7 2nd/ENTER: Select \xB7 APPS: Check";
static const char *str_keyHelp2 =
	"CLEAR: New game \xB7 BkSp: Cancel \xB7 ESC: Quit";
#define STR_OPTIONSKEYHELP_LINES     8
static const char *str_optionsKeyHelp[] =
	{"Press \x15\x17\x16 to",
	 "choose:",
	 "\x15: Game type",
	 "\x17: # players",
	 "\x16: Difficulty",
	 "",
	 "Press 2nd/Enter",
	 "to start game"};

/* New game confirmation */
#define STR_NEWGAMECONFIRM_LINES     7
static const char *str_newGameConfirm[] =
	{"Are you sure you",
	 "want to end this",
	 "game and begin a",
	 "new one?",
	 "",
	 "2nd/ENTER = Yes",
	 "ESC       = No"};

///// Graphics constants ///////////////////////////////////////////////////////

/* === Cards === */

#define GPH_CARD_HEIGHT 20

static const unsigned short GPH_cards[][GPH_CARD_HEIGHT] = {
// CARD_OUTLINE
{0xAAA8,0x0004,0x8000,0x0004,0x8000,0x0004,0x8000,0x0004,
 0x8000,0x0004,0x8000,0x0004,0x8000,0x0004,0x8000,0x0004,
 0x8000,0x0004,0x8000,0x5554},
// CARD_1
{0x7FF8,0x9004,0xB004,0x9004,0x9004,0xB804,0x8004,0x8084,
 0x8184,0x8384,0x8784,0x8384,0x8384,0x8384,0x8384,0x8384,
 0x8384,0x87C4,0x8004,0x7FF8},
// CARD_2
{0x7FF8,0xB004,0x8804,0x9004,0xA004,0xB804,0x8004,0x8784,
 0x9FE4,0xB874,0xB874,0x80E4,0x81C4,0x8384,0x8704,0x8E14,
 0x9FF4,0xBFF4,0x8004,0x7FF8},
// CARD_3
{0x7FF8,0xB004,0x8804,0xB004,0x8804,0xB004,0x8004,0x8784,
 0x8FC4,0x9C64,0x8064,0x83C4,0x83E4,0x80F4,0xB874,0xBCF4,
 0x9FE4,0x8FC4,0x8004,0x7FF8},
// CARD_4
{0x7FF8,0x8804,0x9804,0xA804,0xBC04,0x8804,0x8004,0x8044,
 0x80C4,0x81C4,0x83C4,0x87C4,0x8DC4,0x99D4,0xBFF4,0xBFF4,
 0x81C4,0x83E4,0x8004,0x7FF8},
// CARD_5
{0x7FF8,0xB804,0xA004,0xB004,0x8804,0xB004,0x8004,0x9FF4,
 0x9FF4,0x9C14,0xBC04,0xBF04,0xBFC4,0x80F4,0x9874,0xBCF4,
 0x9FE4,0x8784,0x8004,0x7FF8},
// CARD_6
{0x7FF8,0x9004,0xA004,0xB804,0xA804,0xB804,0x8004,0x80E4,
 0x81C4,0x8384,0x8704,0x8FC4,0x9FE4,0xBCF4,0xB874,0xBCF4,
 0x9FE4,0x8F84,0x8004,0x7FF8},
// CARD_7
{0x7FF8,0xB804,0x8804,0x9004,0xA004,0xA004,0x8004,0xBFF4,
 0xBFE4,0xA0E4,0x81C4,0x8384,0x8384,0x8704,0x8E04,0x9C04,
 0x9C04,0xBE04,0x8004,0x7FF8},
// CARD_8
{0x7FF8,0xB804,0xA804,0xB804,0xA804,0xB804,0x8004,0x8784,
 0x8FC4,0x9CE4,0x9CE4,0x8FC4,0x9FE4,0xBCF4,0xB874,0xBCF4,
 0x9FE4,0x8FC4,0x8004,0x7FF8},
// CARD_9
{0x7FF8,0xB804,0xA804,0xB804,0x8804,0x9004,0x8004,0x8FC4,
 0x9FE4,0xBCF4,0xB874,0xBCF4,0x9FE4,0x8FC4,0x8384,0x8704,
 0x8E04,0x9C04,0x8004,0x7FF8},
// CARD_10
{0x7FF8,0xA404,0xAA04,0xAA04,0xAA04,0xA404,0x8004,0x8CE4,
 0x9DF4,0xBDB4,0x8DB4,0x8DB4,0x8DB4,0x8DB4,0x8DB4,0x8DB4,
 0x8DF4,0x8CE4,0x8004,0x7FF8},
// CARD_11
{0x7FF8,0xA804,0xA804,0xA804,0xA804,0xA804,0x8004,0x8634,
 0x8E74,0x9EF4,0x8634,0x8634,0x8634,0x8634,0x8634,0x8634,
 0x8634,0x8634,0x8004,0x7FF8},
// CARD_12
{0x7FF8,0xAC04,0xA204,0xA404,0xA804,0xAE04,0x8004,0x98C4,
 0xB9E4,0xFB34,0x9B34,0x9864,0x98C4,0x9984,0x9B04,0x9B14,
 0x9BF4,0x9BF4,0x8004,0x7FF8},
// CARD_SKIPBO
{0x7FF8,0xBC04,0xAA04,0xBC04,0x9A04,0xBC04,0x8004,0x9CE4,
 0xA294,0xA294,0xA094,0x9094,0x88E4,0x8494,0xA294,0xA294,
 0xA294,0x9CE4,0x8004,0x7FF8},
// CARD_BACK
{0x7FF8,0x8004,0xBFF4,0xBFF4,0xA314,0x9D64,0x9D64,0x9F64,
 0xAF64,0xB714,0xBB64,0xBD64,0x9D64,0x9D64,0x9D64,0xA314,
 0xBFF4,0xBFF4,0x8004,0x7FF8},
// CARD_HIGHLIGHTER
{0x0000,0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,
 0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,0x7FF8,
 0x7FF8,0x7FF8,0x7FF8,0x0000},
// CARD_MASK
{0xFFFF,0x8007,0x8007,0x8007,0x8007,0x8007,0x8007,0x8007,
 0x8007,0x8007,0x8007,0x8007,0x8007,0x8007,0x8007,0x8007,
 0x8007,0x8007,0x8007,0xFFFF}
// CARD_X
//{0x0000,0x6018,0x6018,0x3030,0x1860,0x1860,0x0CC0,0x0CC0,
// 0x0780,0x0300,0x0300,0x0780,0x0CC0,0x0CC0,0x1860,0x1860,
// 0x3030,0x6018,0x6018,0x0000}
};

/* === Other small graphic symbols === */

#define GPH_TWELVE_HEIGHT    9
#define GPH_PLAYER_HEIGHT    8
#define GPH_CURSOR_HEIGHT    3

static const unsigned char GPH_twelve[] =
	{0xFE,0x00,0x58,0x44,0x48,0x50,0x5C,0x00,0xFE};
static const unsigned char GPH_player[] =
	//{0x20,0x70,0x70,0x20,0x70,0xF8,0xF8};
	{0x10,0x38,0x38,0x10,0x38,0x7C,0x7C,0x00};
static const unsigned char GPH_cursor[] =
	{0x10,0x38,0x7C};

///// Global variables /////////////////////////////////////////////////////////
unsigned char cursorOn = FALSE, cursorPosition = 0,
	selectedCard = NO_CARD_SELECTED, cursorCurPosX = 0, cursorCurPosY = 0;

////////////////////////////////////////////////////////////////////////////////
void DrawFramework(void) {
	// === Graphics constants ===
	const unsigned char coordinates[] =
		{15, 25, 43, 62, 96};
	// This takes less space than using LCD_WIDTH all the time:
	const unsigned char lcd_width = LCD_WIDTH;

	// === Variables ===
	unsigned char loop;

	// === Labels ===
	ClrScr();
	FontSetSys(F_4x6);
	DrawStr(0, 0, str_keyHelp1, A_NORMAL);
	DrawStr(0, 6, str_keyHelp2, A_NORMAL);

	// === Status divider lines ===
	DrawLine(0, 12, lcd_width, 12, A_NORMAL);
	for (loop = 0; loop < 5; loop++) {
		DrawLine(lcd_width - 50, coordinates[loop], lcd_width,
			coordinates[loop], A_NORMAL);
	}
	DrawLine(lcd_width - 50, 15, lcd_width - 50, 96, A_NORMAL);

	// === Number of players and number of completed building piles sprites ===
	Sprite8(
		lcd_width - 48, 26, GPH_PLAYER_HEIGHT, GPH_player, LCD_MEM, SPRT_OR);
	Sprite8(
		lcd_width - 48, 44, GPH_TWELVE_HEIGHT, GPH_twelve, LCD_MEM, SPRT_XOR);
}

////////////////////////////////////////////////////////////////////////////////
void DrawPlayers(PLAYER *player, unsigned char playerNo) {
	unsigned char loop, lcd_width = LCD_WIDTH, yPos;
	FontSetSys(F_6x8);
	for (loop = PLYR_A; loop <= PLYR_D; loop++) {
		yPos = 64 + (8*loop);
		Sprite8(lcd_width - 49, yPos, GPH_PLAYER_HEIGHT, GPH_player, LCD_MEM,
			SPRT_AND);
		Sprite8(lcd_width - 49, yPos, GPH_PLAYER_HEIGHT, GPH_player, LCD_MEM,
			SPRT_OR);
		printf_xy(lcd_width - 42, yPos, "%c: %4u",
			str_playerChar[loop], player[loop].score);
		if (playerNo == loop)
			ScrRectFill(
				&(SCR_RECT){{lcd_width-49, yPos-1, lcd_width-36, yPos+7}},
				&(SCR_RECT){{0, 0, lcd_width, LCD_HEIGHT}}, A_XOR);
	}
}

////////////////////////////////////////////////////////////////////////////////
void DrawGameStatus(GAME *game) {
	const unsigned char lcd_width = LCD_WIDTH;
	FontSetSys(F_6x8);
	printf_xy(lcd_width - 48, 17, "%-8s",
		str_gametype[game->type]);
	printf_xy(lcd_width - 42, 27, "\xD7%6u",
		game->numPlayers+1);
	printf_xy(lcd_width - 48, 35, "%-8s",
		str_gameDifficulty[game->difficulty]);
	printf_xy(lcd_width - 42, 45, "\xD7%6u",
		game->numCmplBuildPiles);
}

////////////////////////////////////////////////////////////////////////////////
void DrawDrawPile(GAME *game) {
	const unsigned char lcd_width = LCD_WIDTH;
	FontSetSys(F_6x8);
	printf_xy(lcd_width - 48, 54, "Draw\xD7%3u",
		SizeOfPile(game->drawPile, DECK_SIZE));
	if (selectedCard == DRAW_PILE) {
		ScrRectFill(
			&(SCR_RECT){{lcd_width-49, 54, lcd_width, 61}},
			&(SCR_RECT){{0,0,lcd_width,LCD_HEIGHT}}, A_XOR);
	} else {
		DrawLine(lcd_width-49, 54, lcd_width-49, 61, A_REVERSE);
	}
}

////////////////////////////////////////////////////////////////////////////////
void DrawCompressedPile(unsigned char x, unsigned char y, unsigned char *pile, unsigned char pileSize) {
	unsigned char topCardY = y + pileSize/5;
	if (pileSize > 0) pileSize--;
	/* Draw stack of cards above */
	if (topCardY > y)
		DrawLine(x+1, y, x+12, y, A_NORMAL);
		ScrRectFill(
			&(SCR_RECT){{x, y+1, x+13, topCardY}},
			&(SCR_RECT){{0,0,LCD_WIDTH,LCD_HEIGHT}}, A_NORMAL);
	/* Draw topmost card */
	Sprite16(x, topCardY, GPH_CARD_HEIGHT, GPH_cards[pile[pileSize]], LCD_MEM,
		SPRT_OR);
}

////////////////////////////////////////////////////////////////////////////////
void DrawBuildingPiles(GAME *game) {
	unsigned char pile, card, pileSize, cardY = 15;
	// Erase the exisitng building piles first
	ScrRectFill(&(SCR_RECT){{0, 15, 15*4, 15+3+GPH_CARD_HEIGHT}},
		&(SCR_RECT){{0,0,LCD_WIDTH,LCD_HEIGHT}}, A_REVERSE);
	for (pile = 0; pile < 4; pile++) {

		/* Get the size of a building pile and draw it */
		pileSize = SizeOfPile(game->buildPile[pile], 12);
		DrawCompressedPile(15*pile, 15, game->buildPile[pile], pileSize);
		/* Skip-bo cards require special handling (the number they replace
		   should be shown in the lower portion of the card instead of the
		   Skip-bo symbol), hence the following code */
		card = game->buildPile[pile][pileSize>0 ? pileSize-1 : 0];
		cardY = 15 + pileSize/5;
		if (card == CARD_SKIPBO) {
			/* Draw the appropriately numbered card in place of Skip-bo,
			   then replace the upper (small) number in the upper-left
			   corner of the card with the Skip-bo symbol */
			Sprite16(15*pile, cardY, GPH_CARD_HEIGHT, GPH_cards[CARD_MASK],
				LCD_MEM, SPRT_AND);
			Sprite16(15*pile, cardY, GPH_CARD_HEIGHT,
				GPH_cards[BuildEffectiveCard(pile, game)], LCD_MEM, SPRT_OR);
			Sprite16(15*pile, cardY, 6, GPH_cards[CARD_MASK], LCD_MEM,
				SPRT_AND);
			Sprite16(15*pile, cardY, 6, GPH_cards[CARD_SKIPBO], LCD_MEM,
				SPRT_OR);
		}
		// Invert the card if it's selected
		if (selectedCard == BUILD_PILE_1 + pile) {
			Sprite16(15*pile, cardY, GPH_CARD_HEIGHT,
				GPH_cards[CARD_HIGHLIGHTER], LCD_MEM, SPRT_XOR);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
void DrawStockPile(PLAYER *player, unsigned char playerNo) {
	unsigned char pileSize = SizeOfPile(player[playerNo].stockPile, 30);

	/* Erase the stock pile first */
	ScrRectFill(&(SCR_RECT){{65, 15, 65+16, 15+6+GPH_CARD_HEIGHT}},
		&(SCR_RECT){{0, 0, LCD_WIDTH, LCD_HEIGHT}}, A_REVERSE);

	/* Draw the cards */
	DrawCompressedPile(65, 15, player[playerNo].stockPile, pileSize);

	// Invert the card if it's selected
	if (selectedCard == STOCK_PILE)
		Sprite16(65, 15+pileSize/5, GPH_CARD_HEIGHT,
			GPH_cards[CARD_HIGHLIGHTER], LCD_MEM, SPRT_XOR);

	/* Draw number of stock pile cards remaining */
	FontSetSys(F_6x8);
	printf_xy(84,21,"\xD7%2u",pileSize);

}

////////////////////////////////////////////////////////////////////////////////
void DrawDiscardPiles(PLAYER *player, unsigned char playerNo) {
	unsigned char pile, pileSize, cardSpacing, loop;
	signed char yAdjustment;

	/* Erase the existing discard piles first */
	ScrRectFill(&(SCR_RECT){{0, 45, 15*4, LCD_HEIGHT}},
		&(SCR_RECT){{0, 45, 15*4, LCD_HEIGHT}}, A_REVERSE);

	for (pile = 0; pile < 4; pile++) {
		/* Get the size of a discard pile and draw all but the
		   topmost card */
		pileSize = SizeOfPile(player[playerNo].discardPile[pile],
			DISCARD_PILE_SIZE);
		cardSpacing = (pileSize <= 5 ? 7 : 6);
		yAdjustment = 0;
		if (pileSize > 6) {
			/* If stack is more than 6 cards, show an extra row
			   of pixels at the top to help indicate that not all
			   cards are shown */
			DrawLine(15*pile+1, 45, 15*pile+12, 45, A_NORMAL);
			yAdjustment = 1;
		}
		for (loop = 0; loop+1 < pileSize; loop++) {
			/* Draw the stack of cards */
			if ((pileSize - loop) <= 6) {
				/* Only the topmost 6 cards of each discard pile
				   will be visible due to screen space
				   limitations */
				Sprite16(15*pile, 45+loop*cardSpacing+yAdjustment,
					cardSpacing+1,
					GPH_cards[player[playerNo].discardPile[pile][loop]],
					LCD_MEM, SPRT_OR);
			} else {
				yAdjustment -= cardSpacing;
			}
		}
		/* Finally, reprint the topmost card */
		Sprite16(15*pile, 45+loop*cardSpacing+yAdjustment, GPH_CARD_HEIGHT,
			GPH_cards[player[playerNo].discardPile[pile][loop]],
			LCD_MEM, SPRT_OR);
		// Invert the card if it's selected
		if (selectedCard == DISC_PILE_1 + pile)
			Sprite16(15*pile, 45+loop*cardSpacing+yAdjustment,
				GPH_CARD_HEIGHT, GPH_cards[CARD_HIGHLIGHTER],
				LCD_MEM, SPRT_XOR);
	}

}

////////////////////////////////////////////////////////////////////////////////
void DrawHand(PLAYER *player, unsigned char playerNo) {
	unsigned char loop, card;

	// Erase hand
	ScrRectFill(&(SCR_RECT){{64, 45, 64+6*7, 44+GPH_CARD_HEIGHT}},
		&(SCR_RECT){{0, 0, LCD_WIDTH, LCD_HEIGHT}}, A_REVERSE);

	// Draw hand cards
	for (loop = 0; loop < 5; loop++) {
		if (player[playerNo].hand[loop] >= CARD_1 &&
			player[playerNo].hand[loop] <= CARD_SKIPBO) {
			Sprite16(64+loop*7, 45, GPH_CARD_HEIGHT,
				GPH_cards[CARD_MASK], LCD_MEM, SPRT_AND);
#ifndef DEBUG
			if (playerNo == PLYR_A) {
				card = player[playerNo].hand[loop];
			} else {
				/* Hide computer players' cards */
				card = CARD_BACK;
			}
#else
			card = player[playerNo].hand[loop];
#endif
			Sprite16(64+loop*7, 45, GPH_CARD_HEIGHT,
				GPH_cards[card], LCD_MEM, SPRT_OR);
			// Invert if selected
			if (selectedCard == HAND_1+loop)
				Sprite16(64+loop*7, 45, GPH_CARD_HEIGHT,
				GPH_cards[CARD_HIGHLIGHTER], LCD_MEM,
				SPRT_XOR);
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
void DrawAllCards(GAME *game, PLAYER *player, unsigned char playerNo) {
	DrawDrawPile(game);
	DrawBuildingPiles(game);
	DrawStockPile(player, playerNo);
	DrawDiscardPiles(player, playerNo);
	DrawHand(player, playerNo);
}

////////////////////////////////////////////////////////////////////////////////
void DrawWholeScreen(GAME *game, PLAYER *player, unsigned char playerNo) {
	DrawFramework();
	DrawGameStatus(game);
	DrawPlayers(player, playerNo);
	DrawAllCards(game, player, playerNo);
}

////////////////////////////////////////////////////////////////////////////////
void EraseCardArea(void) {
	unsigned char lcd_width = LCD_WIDTH, lcd_height = LCD_HEIGHT;
	ScrRectFill(&(SCR_RECT){{0, 13, lcd_width - 51, lcd_height}},
		&(SCR_RECT){{0,0,lcd_width,lcd_height}}, A_REVERSE);
}

void PrintGameOptionsHelp(void) {
	unsigned char i, lcd_width = LCD_WIDTH;

	EraseCardArea();

	/* Print help text */
	FontSetSys(F_4x6);
	printf_xy(1, 15, str_author);
	DrawLine(0, 21, lcd_width - 52, 21, A_NORMAL);
	FontSetSys(F_6x8);
	for (i = 0; i < STR_OPTIONSKEYHELP_LINES; i++) {
		printf_xy(9, i*8+28, str_optionsKeyHelp[i]);
	}
}

void PrintNewGameConfirmation(void) {
	unsigned char i;

	EraseCardArea();

	FontSetSys(F_6x8);
	for (i = 0; i < STR_NEWGAMECONFIRM_LINES; i++) {
		printf_xy(6, i*8+28, str_newGameConfirm[i]);
	}
}

////////////////////////////////////////////////////////////////////////////////
void CursorOff(void) {
	if (cursorOn) {
		Sprite8(cursorCurPosX, cursorCurPosY, GPH_CURSOR_HEIGHT,
			GPH_cursor, LCD_MEM, SPRT_XOR);
		cursorOn = FALSE;
	}
}

void CursorOn(GAME *game, PLAYER *player, unsigned char playerNo) {
	static const unsigned char cursorPosX[] =
		{4,19,34,49,69,4,19,34,49,65,72,79,86,93};
	static const unsigned char cursorPosY[] =
		{36,36,36,36,36,66,66,66,66,66,66,66,66,66};

	unsigned char yAdjustment = 0, pileSize = 0;

	if (!cursorOn) {
		/* Determine cursor Y adjustment (so it will be placed just
		   at the bottom of the deck of cards */
		if (cursorPosition <= BUILD_PILE_4) {
			yAdjustment = SizeOfPile(game->buildPile[cursorPosition], 12) / 5;
		} else if (cursorPosition == STOCK_PILE) {
			yAdjustment = SizeOfPile(player[playerNo].stockPile, 30) / 5;
		} else if (cursorPosition >= DISC_PILE_1 &&
		cursorPosition <= DISC_PILE_4) {
			pileSize = SizeOfPile(
				player[playerNo].discardPile[cursorPosition - DISC_PILE_1],
				DISCARD_PILE_SIZE);
			if (pileSize <= 5) {
				yAdjustment = pileSize*7;
			} else {
				yAdjustment = 1 + pileSize*6;
			}
			if (yAdjustment > 38) yAdjustment = 38;
			if (yAdjustment > 0) yAdjustment -= 7;
		}

		// Remember current cursor positions
		cursorCurPosX = cursorPosX[cursorPosition];
		cursorCurPosY = cursorPosY[cursorPosition] + yAdjustment;

		// Draw cursor
		Sprite8(cursorCurPosX, cursorCurPosY, GPH_CURSOR_HEIGHT,
			GPH_cursor, LCD_MEM, SPRT_XOR);

		cursorOn = TRUE;
	}
}

void CursorToggle(GAME *game, PLAYER *player, unsigned char playerNo) {
	if (cursorOn) CursorOff(); else CursorOn(game, player, playerNo);
}

////////////////////////////////////////////////////////////////////////////////
void UpdateSelection(GAME *game, PLAYER *player, unsigned char playerNo) {
	if (selectedCard <= BUILD_PILE_4) {
		DrawBuildingPiles(game);
	} else if (selectedCard == STOCK_PILE) {
		DrawStockPile(player, playerNo);
	} else if (selectedCard > STOCK_PILE && selectedCard <= DISC_PILE_4) {
		DrawDiscardPiles(player, playerNo);
	} else if (selectedCard >= HAND_1 && selectedCard <= HAND_5) {
		DrawHand(player, playerNo);
	} else if (selectedCard == DRAW_PILE) {
		DrawDrawPile(game);
	} else if (selectedCard == NO_CARD_SELECTED) {
		DrawAllCards(game, player, playerNo);
	}
}


////////////////////////////////////////////////////////////////////////////////
void AnimateCard(unsigned char pile, unsigned char playerNo, GAME *game,
	PLAYER *player) {
	unsigned char saveSelectedCard = selectedCard;
	selectedCard = pile;
	UpdateSelection(game, player, playerNo);
	Delay(CARD_ANIMATE_SPEED);
	selectedCard = saveSelectedCard;
	UpdateSelection(game, player, playerNo);
}

////////////////////////////////////////////////////////////////////////////////
void UpdatePileIndicator(unsigned char pile) {
	unsigned char pileType = 0;
	const SCR_RECT screen = {{0,0,LCD_WIDTH,LCD_HEIGHT}};
	if (pile <= BUILD_PILE_4) {
		pileType = 0;
	} else if (pile == STOCK_PILE) {
		pileType = 1;
	} else if (pile <= DISC_PILE_4) {
		pileType = 2;
	} else if (pile <= HAND_5) {
		pileType = 3;
	} else {
		pileType = 4;
	}

	ScrRectFill(&(SCR_RECT){{64, 48+GPH_CARD_HEIGHT+GPH_CURSOR_HEIGHT,
		64+6*7, 58+GPH_CARD_HEIGHT+GPH_CURSOR_HEIGHT}}, &screen, A_REVERSE);
	FontSetSys(F_4x6);
	if (pileType < 4) {
		printf_xy(64, 48+GPH_CARD_HEIGHT+GPH_CURSOR_HEIGHT,
			str_pileNames[pileType]);
		if (pileType != 3) printf_xy(64, 54+GPH_CARD_HEIGHT+GPH_CURSOR_HEIGHT,
			str_pile);
	}
}

////////////////////////////////////////////////////////////////////////////////
void ShuffleAnimation(void) {
	unsigned char lcd_width = LCD_WIDTH, loop;
	DrawLine(0, 44, lcd_width - 51, 44, A_NORMAL);
	DrawLine(0, 54, lcd_width - 51, 54, A_NORMAL);
	FontSetSys(F_6x8);

	// Flashing text "SHUFFLING"
	for (loop = 0; loop < 8; loop++) {
		ScrRectFill(&(SCR_RECT){{0, 45, lcd_width - 51, 53}},
			&(SCR_RECT){{0, 0, LCD_WIDTH, LCD_HEIGHT}}, A_REVERSE);
		Delay(10);
		DrawStr(27, 46, str_shuffling, A_NORMAL);
		Delay(10);
	}

	ScrRectFill(&(SCR_RECT){{0, 44, lcd_width - 51, 54}},
		&(SCR_RECT){{0, 0, LCD_WIDTH, LCD_HEIGHT}}, A_REVERSE);
}

////////////////////////////////////////////////////////////////////////////////
void CompletedBuildPileAnimation(GAME *game) {
	unsigned char loop;
	for (loop = 0; loop < 3; loop++) {
		printf_xy(LCD_WIDTH - 36, 45, "      ");
		Delay(50);
		printf_xy(LCD_WIDTH - 42, 45, "\xD7%6u", game->numCmplBuildPiles);
		Delay(50);
	}
}

////////////////////////////////////////////////////////////////////////////////
void BuildPileFlash(unsigned char pileNo, GAME *game) {
	unsigned char saveCard[4], loop, loop2;

	/* Back up cards that will temporarily be changed */
	for (loop = 0; loop <= 3; loop++) {
		saveCard[loop] = game->buildPile[loop][0];
	}

	/* Do animation */
	for (loop = 0; loop < 6; loop++) {
		for (loop2 = (pileNo > 3 ? 0 : pileNo);
		loop2 <= (pileNo > 3 ? 3 : pileNo); loop2++) {
			if (loop % 2) {
				game->buildPile[loop2][0] = saveCard[loop2];
			} else {
				game->buildPile[loop2][0] = CARD_OUTLINE;
			}
		}
		DrawBuildingPiles(game);
		Delay(50);
	}
}

////////////////////////////////////////////////////////////////////////////////
void DealStockPileAnimation(unsigned char numCards) {
	unsigned char i;

	/* Set medium-sized font */
	FontSetSys(F_6x8);

	/* Erase existing stock pile */
	ScrRectFill(&(SCR_RECT){{65, 15, 65+16, 15+6+GPH_CARD_HEIGHT}},
		&(SCR_RECT){{0, 0, LCD_WIDTH, LCD_HEIGHT}}, A_REVERSE);

	for (i = 0; i < numCards; i++) {
		/* Draw next card on stack */
		Sprite16(65, 15+(i/5), GPH_CARD_HEIGHT, GPH_cards[CARD_MASK],
			LCD_MEM, SPRT_AND);
		Sprite16(65, 15+(i/5), GPH_CARD_HEIGHT, GPH_cards[CARD_BACK],
			LCD_MEM, SPRT_OR);
		/* Update number of stock pile cards */
		printf_xy(84,21,"\xD7%2u",i+1);
		/* Delay */
		Delay(5);
	}
}

////////////////////////////////////////////////////////////////////////////////
void Delay(unsigned long length) {
	unsigned long doneTime;
	doneTime = FiftyMsecTick + length;
	while (FiftyMsecTick < doneTime) idle();
}
