/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* input.c
   Keyboard input/game control functions */

#include <compat.h>
#include <kbd.h>
#include <system.h>

#include "graphics.h"
#include "skipbo.h"
#include "input.h"

////////////////////////////////////////////////////////////////////////////////
unsigned char GetKeyEvent(unsigned char cursor, unsigned char playerNo,
GAME *game, PLAYER *player) {
	unsigned char lastKeyReleased = FALSE, keyEv = KE_NO_KEY,
		keyEvOld = KE_NO_KEY;
	unsigned long cursorTimer = FiftyMsecTick, apdTimer = cursorTimer,
		keyDebounceTimer = cursorTimer;

	/* Reset card cursor */
	if (cursor)
		CursorOn(game, player, playerNo);

	/* Wait until the last key (if any) is released and a new key is
	   pressed */
	while (!(lastKeyReleased && keyEv && (FiftyMsecTick - keyDebounceTimer) >=
	KBD_DEBOUNCE_TIME)) {
		idle();

		if (!lastKeyReleased) {
			/* Waiting for last key to be released */
			if (!GetKeyNow()) lastKeyReleased = TRUE;
		} else {
			/* Waiting for new key */
			keyEv = GetKeyNow();
			if (keyEv != keyEvOld) keyDebounceTimer = FiftyMsecTick;
			keyEvOld = keyEv;
		}

		/* Update card cursor */
		if ( cursor && (cursorTimer != FiftyMsecTick) &&
		!((FiftyMsecTick - cursorTimer) % CURSOR_FLASH_SPEED) ) {
			CursorToggle(game, player, playerNo);
			cursorTimer = FiftyMsecTick;
		}

		/* APD timer (shuts calculator off after approx. five minutes) */
		if ((FiftyMsecTick - apdTimer) > APD_TIME) {
			off();
			apdTimer = FiftyMsecTick;
		}
	}

	return keyEv;
}

////////////////////////////////////////////////////////////////////////////////
unsigned char GetKeyNow(void) {
	BEGIN_KEYTEST
		if (_keytest_optimized (RR_LEFT)) return KE_LEFT;
		if (_keytest_optimized (RR_RIGHT)) return KE_RIGHT;
		if (_keytest_optimized (RR_UP)) return KE_UP;
		if (_keytest_optimized (RR_DOWN)) return KE_DOWN;
		if (_keytest_optimized (RR_2ND) ||
			_keytest_optimized (RR_ENTER)) return KE_SELECT;
		if (_keytest_optimized (RR_CLEAR)) return KE_NEW_GAME;
		if (_keytest_optimized (RR_BCKSPC)) return KE_CANCEL;
		if (_keytest_optimized (RR_HOME)) return KE_OFF;
		if (_keytest_optimized (RR_APPS)) return KE_CHECK;
		if (_keytest_optimized (RR_ESC)) return KE_QUIT;
	END_KEYTEST
	return KE_NO_KEY;
}
