/*  TI-89 Skip-Bo clone
 *    Copyright (C) 2009 Travis Evans
 *   
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, version 3.
 *   
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *   
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* input.h
   Defines user-interface data such as arrow cursor positions and key press
   event codes */

#ifndef _INPUT_H
#define _INPUT_H

#define APD_TIME             100*60*5  // Approx. five minutes
#define CURSOR_FLASH_SPEED   50
#define KBD_DEBOUNCE_TIME    5

#include "data.h"

// Key event codes
enum KEY_PRESSES {
	KE_NO_KEY, KE_LEFT, KE_RIGHT, KE_UP, KE_DOWN, KE_SELECT, KE_CHECK,
	KE_CANCEL, KE_OFF, KE_NEW_GAME, KE_QUIT};

///// Functions ////////////////////////////////////////////////////////////////

/* GetKeyNow
   Does a low-level reading of the keypad and returns any code received
   immediately (even if no key was pressed)
   This routine is called by GetKeyEvent() */
unsigned char GetKeyNow(void);

/* Wait for a key press and then return it
   cursor: TRUE if card selection cursor should be shown and blinking
   playerNo: The player whose playing area is currently shown on the screen
   game and player struct pointers must be valid if cursor is TRUE.  Otherwise,
   they can be simply passed as NULL. */
unsigned char GetKeyEvent(unsigned char cursor, unsigned char playerNo,
GAME *game, PLAYER *player);

#endif
